﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using NiVE2.Plugin.Interface;
using NiVE2.Plugin.Utils;
using NiVE2.Utils;

namespace ruche.nive2.effects
{
    /// <summary>
    /// 整数値配列プロパティ値を設定するNiVE2プロパティエディットコントロールクラス。
    /// </summary>
    public class IntArrayPropertyEditControl : PropertyEditControlBase
    {
        private TextBox textArray;
        private Label labelArray;

        /// <summary>
        /// 整数値のインクリメント幅。
        /// </summary>
        private int _increment;

        /// <summary>
        /// 既定のプロパティ値。
        /// </summary>
        private IntArrayProperty _defaultProp;

        /// <summary>
        /// マウスの左ボタンが投下中であるか否か。
        /// </summary>
        private bool _mousePushing = false;

        /// <summary>
        /// マウスカーソルが左ボタン投下後に動いたか否か。
        /// </summary>
        private bool _mouseMoved = false;

        /// <summary>
        /// マウスの左ボタン投下時のカーソルX位置。
        /// </summary>
        private int _baseX = 0;

        /// <summary>
        /// マウスの左ボタン投下後の直前のカーソルX位置。
        /// </summary>
        private int _prevX = 0;

        /// <summary>
        /// マウスの左ボタン投下時の整数配列値。
        /// </summary>
        private int[] _baseValues = null;

        /// <summary>
        /// コンストラクタ。
        /// </summary>
        /// <param name="name">対応するプロパティ名。</param>
        /// <param name="increment">整数値のインクリメント幅。</param>
        public IntArrayPropertyEditControl(string name, int increment)
        {
            PropertyName = name;
            LabelName = name;
            _increment = increment;

            InitializeComponent();
        }

        /// <summary>
        /// テキスト編集中であるか否かを取得する。
        /// </summary>
        private bool IsEditing
        {
            get { return textArray.Visible; }
        }

        /// <summary>
        /// 新しい整数値配列プロパティインスタンスを作成する。
        /// </summary>
        /// <returns>整数値配列プロパティインスタンス。</returns>
        private IntArrayProperty CreateProperty()
        {
            return (_defaultProp == null) ?
                (new IntArrayProperty(PropertyName, null, 0)) :
                ((IntArrayProperty)_defaultProp.Copy());
        }

        /// <summary>
        /// 時刻値の編集を開始する。
        /// </summary>
        private void BeginEdit()
        {
            if (!IsEditing)
            {
                // 編集モード開始
                labelArray.Visible = false;
                textArray.Visible = true;

                // 全選択してフォーカス
                textArray.SelectAll();
                textArray.Select();
            }
        }

        /// <summary>
        /// 時刻値の編集を完了する。
        /// </summary>
        private void EndEdit()
        {
            if (IsEditing)
            {
                if (textArray.Text != labelArray.Text)
                {
                    // 値が変わっていたらプロパティ更新
                    IntArrayProperty prop = CreateProperty();
                    prop.SetCsvString(textArray.Text);
                    SetProperty(prop);
                }

                // 編集モード終了
                textArray.Visible = false;
                labelArray.Visible = true;
            }
        }

        /// <summary>
        /// 時刻値の編集をキャンセルする。
        /// </summary>
        private void CancelEdit()
        {
            if (IsEditing)
            {
                // ラベルのテキストに戻す
                textArray.Text = labelArray.Text;

                // 編集モード終了
                textArray.Visible = false;
                labelArray.Visible = true;
            }
        }

        #region PropertyEditControlBase メンバ

        public override int SpliterDistance
        {
            get
            {
                return base.SpliterDistance;
            }
            set
            {
                base.SpliterDistance = value;

                // コントロールの位置合わせを行う
                textArray.Left = value + Util.PropertyEditControlLeftDistance;
                textArray.Width = Math.Max(this.Width - value - 72, 20);
                labelArray.Left = textArray.Left;
            }
        }

        public override Type UseProperetyType()
        {
            return typeof(IntArrayProperty);
        }

        public override void ChangeProperty(PropertyBase property)
        {
            IntArrayProperty prop = property as IntArrayProperty;
            if (prop != null)
            {
                // 表示文字列更新
                string text = prop.GetCsvString();
                textArray.Text = text;
                labelArray.Text = text;
            }

            base.ChangeProperty(property);
        }

        public override void SetDefaultProperty(PropertyBase property)
        {
            if (property is IntArrayProperty)
            {
                // 既定のプロパティ値設定
                _defaultProp = (IntArrayProperty)property.Copy();
            }

            base.SetDefaultProperty(property);
        }

        #endregion

        private void InitializeComponent()
        {
            this.textArray = new System.Windows.Forms.TextBox();
            this.labelArray = new System.Windows.Forms.Label();
            this.SuspendLayout();
            // 
            // textArray
            // 
            this.textArray.BorderStyle = System.Windows.Forms.BorderStyle.None;
            this.textArray.Font = new System.Drawing.Font("ＭＳ ゴシック", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(128)));
            this.textArray.Location = new System.Drawing.Point(114, 3);
            this.textArray.Name = "textArray";
            this.textArray.Size = new System.Drawing.Size(90, 12);
            this.textArray.TabIndex = 2;
            this.textArray.Text = "0";
            this.textArray.Visible = false;
            this.textArray.KeyDown += new System.Windows.Forms.KeyEventHandler(this.textArray_KeyDown);
            this.textArray.Leave += new System.EventHandler(this.textArray_Leave);
            this.textArray.KeyPress += new System.Windows.Forms.KeyPressEventHandler(this.textArray_KeyPress);
            // 
            // labelArray
            // 
            this.labelArray.AutoSize = true;
            this.labelArray.Cursor = System.Windows.Forms.Cursors.SizeWE;
            this.labelArray.Font = new System.Drawing.Font("ＭＳ ゴシック", 9F, System.Drawing.FontStyle.Underline, System.Drawing.GraphicsUnit.Point, ((byte)(128)));
            this.labelArray.Location = new System.Drawing.Point(114, 3);
            this.labelArray.Name = "labelArray";
            this.labelArray.Size = new System.Drawing.Size(11, 12);
            this.labelArray.TabIndex = 3;
            this.labelArray.Text = "0";
            this.labelArray.MouseMove += new System.Windows.Forms.MouseEventHandler(this.labelArray_MouseMove);
            this.labelArray.MouseDown += new System.Windows.Forms.MouseEventHandler(this.labelArray_MouseDown);
            this.labelArray.MouseUp += new System.Windows.Forms.MouseEventHandler(this.labelArray_MouseUp);
            // 
            // IntArrayPropertyEditControl
            // 
            this.AutoScaleDimensions = new System.Drawing.SizeF(6F, 12F);
            this.Controls.Add(this.textArray);
            this.Controls.Add(this.labelArray);
            this.Name = "IntArrayPropertyEditControl";
            this.Controls.SetChildIndex(this.labelArray, 0);
            this.Controls.SetChildIndex(this.textArray, 0);
            this.ResumeLayout(false);
            this.PerformLayout();

        }

        private void labelArray_MouseDown(object sender, MouseEventArgs e)
        {
            if (!IsEditing)
            {
                // 左ボタン投下
                _mousePushing = true;
                _mouseMoved = false;
                _baseX = e.X;
                _prevX = _baseX;
                IntArrayProperty prop = CreateProperty();
                prop.SetCsvString(labelArray.Text);
                _baseValues = prop.Values;
            }
        }

        private void labelArray_MouseMove(object sender, MouseEventArgs e)
        {
            if (!IsEditing && _mousePushing)
            {
                if (e.X != _prevX)
                {
                    // X位置更新
                    _prevX = e.X;

                    // ドラッグしたことにする
                    _mouseMoved = true;

                    // プロパティ設定
                    IntArrayProperty prop = CreateProperty();
                    prop.Values = _baseValues;
                    int incVal = (e.X - _baseX) * _increment;
                    for (int i = 0; i < prop.Count; ++i)
                    {
                        prop[i] += incVal;
                    }
                    SetProperty(prop);
                }
            }
        }

        private void labelArray_MouseUp(object sender, MouseEventArgs e)
        {
            if (!IsEditing && _mousePushing)
            {
                // 左ボタン投下完了
                _mousePushing = false;

                if (
                    !_mouseMoved &&
                    e.X >= 0 && e.X < labelArray.Width &&
                    e.Y >= 0 && e.Y < labelArray.Height)
                {
                    // ドラッグしておらず範囲内なら編集モード開始
                    BeginEdit();
                }
            }
        }

        private void textArray_Leave(object sender, EventArgs e)
        {
            if (IsEditing)
            {
                // フォーカスが離れたら編集完了
                EndEdit();
            }
        }

        private void textArray_KeyDown(object sender, KeyEventArgs e)
        {
            if (IsEditing)
            {
                switch (e.KeyCode)
                {
                    case Keys.Enter:
                        // Enterキー投下で編集完了
                        EndEdit();
                        break;

                    case Keys.Escape:
                        // Escキー投下で編集キャンセル
                        CancelEdit();
                        break;
                }
            }
        }

        private void textArray_KeyPress(object sender, KeyPressEventArgs e)
        {
            // 編集終了済みならキー入力を受け付けない
            if (!IsEditing)
            {
                e.Handled = true;
            }
        }
    }
}
